/*
 * Copyright 2016 - 2025 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include "context.hpp"
#include "gfx.hpp"

namespace nostalgia::gfx {

void safeDelete(Context *ctx) noexcept {
	delete ctx;
}

Context::Context(turbine::Context &tctx, InitParams const&params) noexcept:
	turbineCtx(tctx),
	spriteBlocks(params.glSpriteCount, params.glBlocksPerSprite),
	drawer(*this),
	spriteCount(params.glSpriteCount),
	blocksPerSprite(params.glBlocksPerSprite) {
}

Context::~Context() noexcept {
	shutdownGfx(*this);
}

ox::Result<ox::UPtr<Context>> init(turbine::Context &tctx, InitParams const&params) noexcept {
	auto ctx = ox::make_unique<Context>(tctx, params);
	OX_RETURN_ERROR(initGfx(*ctx, params));
	return ox::UPtr<Context>(ctx.release());
}

keel::Context &keelCtx(Context &ctx) noexcept {
	return turbine::keelCtx(ctx.turbineCtx);
}

turbine::Context &turbineCtx(Context &ctx) noexcept {
	return ctx.turbineCtx;
}

}
