/*
 * Copyright 2016 - 2025 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#pragma once

#include <ox/event/signal.hpp>
#include <ox/std/memory.hpp>

#include <studio/filetreemodel.hpp>
#include <studio/widget.hpp>

namespace studio {

class ProjectExplorer final: public Widget, public FileExplorer {
	private:
		ox::UPtr<FileTreeModel> m_treeModel;
		turbine::Context &m_ctx;

	public:
		// slots
		ox::Signal<ox::Error(ox::StringViewCR)> fileChosen;
		ox::Signal<ox::Error(ox::StringViewCR)> addItem;
		ox::Signal<ox::Error(ox::StringViewCR)> addDir;
		ox::Signal<ox::Error(ox::StringViewCR)> deleteItem;

		explicit ProjectExplorer(turbine::Context &ctx) noexcept;

		void draw(StudioContext &ctx) noexcept override;

		void setModel(ox::UPtr<FileTreeModel> &&model) noexcept;

		ox::Error refreshProjectTreeModel(ox::StringViewCR = {}) noexcept;

		[[nodiscard]]
		ox::FileSystem *romFs() noexcept {
			return rom(m_ctx);
		}

	protected:
		void fileOpened(ox::StringViewCR path) const noexcept override;

		void drawFileContextMenu(ox::StringViewCR path) const noexcept override;

		void drawDirContextMenu(ox::StringViewCR path) const noexcept override;

};

}
