/*
 * Copyright 2016 - 2025 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include <imgui.h>

#include <studio/dragdrop.hpp>
#include <studio/imguiutil.hpp>

#include <studio/filetreemodel.hpp>

namespace studio {

FileTreeModel::FileTreeModel(
	FileExplorer &explorer,
	ox::StringParam name,
	FileTreeModel *parent) noexcept:
	m_explorer{explorer},
	m_parent{parent},
	m_name{std::move(name)} {
}

void FileTreeModel::draw(turbine::Context &tctx) const noexcept {
	constexpr auto dirFlags = ImGuiTreeNodeFlags_OpenOnArrow | ImGuiTreeNodeFlags_OpenOnDoubleClick;
	if (!m_children.empty()) {
		if (ImGui::TreeNodeEx(m_name.c_str(), dirFlags)) {
            m_explorer.drawDirContextMenu(m_fullPath);
			for (auto const&child : m_children) {
				child->draw(tctx);
			}
			ImGui::TreePop();
		} else {
			ig::IDStackItem const idStackItem{m_name};
            m_explorer.drawDirContextMenu(m_fullPath);
		}
	} else {
		if (ImGui::TreeNodeEx(m_imguiNodeName.c_str(), ImGuiTreeNodeFlags_Leaf)) {
			if (ImGui::IsItemHovered() && ImGui::IsMouseDoubleClicked(0)) {
				m_explorer.fileOpened(m_fullPath);
			}
			m_explorer.drawFileContextMenu(m_fullPath);
			ImGui::TreePop();
			std::ignore = ig::dragDropSource([this] {
				ImGui::Text("%s", m_name.c_str());
				return ig::setDragDropPayload("FileRef", FileRef{ox::String{m_fullPath}});
			});
		}
	}
}

void FileTreeModel::setChildren(ox::Vector<ox::UPtr<FileTreeModel>> children) noexcept {
	m_children = std::move(children);
}

}
